import pulumi
from pulumi import Output
from pulumi_aws import elasticsearch
from pydantic.dataclasses import dataclass, ConfigDict
from typing import Optional, Mapping

from theia import ResourceOptions
from theia.utils.logger import Logger
from theia._registry import StackRegistry, GlobalTags
from theia.definitions import THEIA_VERSION

logger = Logger("opensearch")


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class NetworkSettings:
    subnets: list[str]
    security_groups: Optional[list[str | Output[str]]] = None


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class DomainSettings:
    version: Optional[str] = None
    instance_type: Optional[str] = "t3.small.elasticsearch"
    instance_count: Optional[int] = 1
    ebs_size: Optional[int] = None
    access_policies: Optional[str] = None


class Domain:
    def __new__(
        cls,
        resource_name: str,
        domain_settings: Optional[DomainSettings] = DomainSettings(),
        network_settings: Optional[NetworkSettings] = None,
        tags: Optional[Mapping[str, str]] = None,
        opts: Optional[ResourceOptions] = None,
    ) -> elasticsearch.Domain:

        name = {"Name": resource_name}
        merged_tags = THEIA_VERSION | GlobalTags.get() | name
        if tags:
            merged_tags = tags | merged_tags

        logger.info(f"creating Elasticsearch domain {resource_name}")

        domain_args = elasticsearch.DomainArgs(
            domain_name=resource_name,
            elasticsearch_version=domain_settings.version,
            ebs_options=(
                None
                if not domain_settings.ebs_size
                else elasticsearch.DomainEbsOptionsArgs(ebs_enabled=True, volume_size=domain_settings.ebs_size)
            ),
            cluster_config=elasticsearch.DomainClusterConfigArgs(
                instance_type=domain_settings.instance_type, instance_count=domain_settings.instance_count
            ),
            vpc_options=(
                None
                if not network_settings
                else elasticsearch.DomainVpcOptionsArgs(
                    subnet_ids=network_settings.subnets, security_group_ids=network_settings.security_groups
                )
            ),
            access_policies=domain_settings.access_policies,
            tags=merged_tags,
        )

        domain_out = elasticsearch.Domain(resource_name=resource_name, args=domain_args, opts=opts)

        pulumi.export(
            f"elasticsearch-domain---{resource_name}",
            {"id": domain_out.id, "arn": domain_out.arn},
        )

        return domain_out

    @staticmethod
    def import_(stack: str, resource_name: str) -> elasticsearch.Domain:
        stack_ref = StackRegistry.get(stack_name=stack)
        domain = stack_ref.get_output(name=f"elasticsearch-domain---{resource_name}")
        return elasticsearch.Domain.get(resource_name=resource_name, id=domain["id"])
