import pulumi
from pulumi_aws import mq
from pydantic.dataclasses import dataclass, ConfigDict
from typing import Optional, List, Mapping, Literal
from dataclasses import field

from theia import ResourceOptions
from theia.utils.logger import Logger
from theia._registry import StackRegistry, GlobalTags
from theia.definitions import THEIA_VERSION, MQ_BROKER_DEFAULT_USER

logger = Logger("mq")



@dataclass
class MQBrokerConfigurations:
    id: Optional[str] = ""
    revision: Optional[str] = ""


@dataclass
class MQBrokerUsers:
    username: Optional[str] = ""
    password: Optional[str] = ""


@dataclass(config=ConfigDict(arbitrary_types_allowed=True))
class MQBrokerSettings:
    engine_version: str
    host_instance_type: Optional[str] = "mq.m5.large"
    configuration: Optional[MQBrokerConfigurations] = None
    subnet_ids: Optional[List[str]] = None
    security_groups: Optional[List[str]] = None
    engine_type: Optional[Literal["ActiveMQ", "RabbitMQ"]] = field(
        default_factory = lambda: ["RabbitMQ"]
    )
    users: Optional[List[MQBrokerUsers]] = field(
        default_factory = lambda: [MQ_BROKER_DEFAULT_USER]
    )
    deployment_mode: Optional[Literal["SINGLE_INSTANCE", "ACTIVE_STANDBY_MULTI_AZ", "CLUSTER_MULTI_AZ"]] = field(
        default_factory = lambda: ["SINGLE_INSTANCE"]
    )



class MQBroker():
    def __new__(
        cls,
        resource_name: str,
        settings: Optional[MQBrokerSettings],
        tags: Optional[Mapping[str, str]] = None,
        opts: Optional[ResourceOptions] = None,
    ) -> mq.BrokerArgs:

        name = {"Name": resource_name}
        merged_tags = THEIA_VERSION | GlobalTags.get() | name
        if tags:
            merged_tags = tags | merged_tags

        if settings.users:
            broker_users = []
            for user in settings.users:
                broker_users.append(
                    mq.BrokerUserArgs(
                        username=user.username,
                        password=user.password
                    )
                )

        if settings.configuration:
            broker_configuration = mq.BrokerConfigurationArgs(
                id=settings.configuration.id,
                revision=settings.configuration.revision
            )

        logger.info(f"creating MQ Broker resource {resource_name}")

        broker_args = mq.BrokerArgs(
            configuration=None if not settings.configuration else broker_configuration,
            engine_type=settings.engine_type,
            engine_version=settings.engine_version,
            host_instance_type=settings.host_instance_type,
            security_groups=settings.security_groups,
            users=None if not settings.users else broker_users,
            deployment_mode=settings.deployment_mode,
            subnet_ids=settings.subnet_ids,
            tags=merged_tags,
        )
        
        broker_out=mq.Broker(
            resource_name=f"{resource_name}",
            args=broker_args,
            opts=opts,
        )

        pulumi.export(
            f"mq-broker---{resource_name}",
            {
                "id": broker_out.id,
                "arn": broker_out.arn,
            },
        )

        return broker_out

    @staticmethod
    def import_(stack: str, resource_name: str) -> mq.Broker:
        stack_ref = StackRegistry.get(stack_name=stack)
        mq = stack_ref.get_output(name=f"mq-broker---{resource_name}")
        return mq.Broker.get(resource_name=resource_name, id=MQBroker["id"])
